/**
 * \file
 *
 * \brief Application CAN Task Management.
 *
 * Copyright (c) 2011 Atmel Corporation. All rights reserved.
 *
 * \asf_license_start
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. The name of Atmel may not be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * 4. This software may only be redistributed and used in connection with an
 *    Atmel microcontroller product.
 *
 * THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
 * EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * \asf_license_stop
 *
 */
#include <stddef.h>
#include <stdio.h>
#include <avr32/io.h>
#include "compiler.h"
#include "board.h"
#include "power_clocks_lib.h"
#include "tc.h"
#include "gpio.h"
#include "pm_uc3c.h"
#include "scif_uc3c.h"
#include "print_funcs.h"
#include "can_task.h"
#include "conf_can_task.h"
#include "conf_demo.h"

//! Local Mob Declaration
can_msg_t mob_ram_ch0[NB_MOB_CHANNEL];
can_msg_t mob_ram_ch1[NB_MOB_CHANNEL];

//! Local function to prepare RX and TX buffers
void can_example_prepare_data_to_send0(void);
void can_example_prepare_data_to_send1(void);
void can_example_prepare_data_to_receive0(void);
void can_example_prepare_data_to_receive1(void);
void can_out_callback_channel0(U8 handle, U8 event);
void can_out_callback_channel00(U8 handle, U8 event);
void can_out_callback_channel1(U8 handle, U8 event);
void can_out_callback_channel11(U8 handle, U8 event);


int counter0 =0;
int counter1 = 0;


//bool ready_send = false;
//uint8_t msg_to_send =0;
void start_can_scheduler(void);
/*! \brief TC interrupt.
 */
#if defined (__GNUC__)
__attribute__((__interrupt__))
#elif defined (__ICCAVR32__)
#pragma handler = TC_INSTANCE_IRQ_GROUP, 1
__interrupt
#endif
static void can_irq(void)
{
	// Clear the interrupt flag. This is a side effect of reading the TC SR.
	tc_read_sr(TC_INSTANCE, TC_CAN_CHANNEL);
	//ready_send = true;
	
	switch(msg_to_send)
	{
		case 0:
			can_example_prepare_data_to_send1();
			msg_to_send = 1;
			break;
		case 1:
			can_example_prepare_data_to_receive0();
			msg_to_send = 0;
			break;
		case 2:
			can_example_prepare_data_to_send0();
			msg_to_send = 3;
			break;
		case 3:
			can_example_prepare_data_to_receive1();
			msg_to_send = 0;
			break;	
		default:
			break;
	}
	
}

volatile avr32_tc_t *tc_can = TC_INSTANCE;


void start_can_scheduler(void)
{
	// Options for waveform genration.
	static const tc_waveform_opt_t WAVEFORM_OPT = {
		.channel  = TC_CAN_CHANNEL,         // Channel selection.
		.bswtrg   = TC_EVT_EFFECT_NOOP, // Software trigger effect on TIOB.
		.beevt    = TC_EVT_EFFECT_NOOP, // External event effect on TIOB.
		.bcpc     = TC_EVT_EFFECT_NOOP, // RC compare effect on TIOB.
		.bcpb     = TC_EVT_EFFECT_NOOP, // RB compare effect on TIOB.
		.aswtrg   = TC_EVT_EFFECT_NOOP, // Software trigger effect on TIOA.
		.aeevt    = TC_EVT_EFFECT_NOOP, // External event effect on TIOA.
		.acpc     = TC_EVT_EFFECT_NOOP, // RC compare effect on TIOA: toggle.
		.acpa     = TC_EVT_EFFECT_NOOP, // RA compare effect on TIOA: toggle.
// Waveform selection: Up mode with automatic trigger(reset) on RC compare.
		.wavsel   = TC_WAVEFORM_SEL_UP_MODE_RC_TRIGGER,
// External event trigger enable.
		.enetrg   = false,
// External event selection.
		.eevt     = 0,
		.eevtedg  = TC_SEL_NO_EDGE,    // External event edge selection.
		.cpcdis   = false,             // Counter disable when RC compare.
		.cpcstop  = false,             // Counter clock stopped with RC compare.
		.burst    = false,             // Burst signal selection.
		.clki     = false,             // Clock inversion.
		.tcclks   = TC_CLOCK_SOURCE_TC5 // Internal source clock 3.
	};

	static const tc_interrupt_t TC_CAN_INTERRUPT = {
		.etrgs = 0,
		.ldrbs = 0,
		.ldras = 0,
		.cpcs  = 1,
		.cpbs  = 0,
		.cpas  = 0,
		.lovrs = 0,
		.covfs = 0
	};
	// Register the TC interrupt handler to the interrupt controller.
	INTC_register_interrupt(&can_irq, TC_CAN_IRQ, AVR32_INTC_INT0);

	// Initialize the timer/counter.
	tc_init_waveform(tc_can, &WAVEFORM_OPT);

	// Set the compare triggers.
	// Remember TC counter is 16-bits, so counting second is not possible
	// with fPBA = 16 MHz. We configure it to count 0.5 s.
	// We want: (1/(fPBA/8)) * RC = 0.5 s, hence RC = (fPBA/128)/2 = 62500 to
	// get an interrupt every 0.5 s.
	tc_write_rc(tc_can, TC_CAN_CHANNEL, (FPBA_HZ / 128)/10); // Set RC value.

	tc_configure_interrupts(tc_can, TC_CAN_CHANNEL, &TC_CAN_INTERRUPT);

	// Start the timer/counter.
	tc_start(tc_can, TC_CAN_CHANNEL);
}



int counter = 0;
//! Boolean for message transmitted on CAN channel 1
volatile bool message_transmitted_on_channel1 = false;
/*! \brief CAN Call Back when message is transmitted
 *
 */
void can_out_callback_channel1(U8 handle, U8 event)
{
	// Transmission Only
	can_mob_free(1,handle);
	message_transmitted_on_channel1 = true;
}

void can_out_callback_channel11(U8 handle, U8 event)
{
	
	// Reception Only
	pCANMOB_rx[0].can_msg->data.u64 = can_get_mob_data(1,handle).u64;
	pCANMOB_rx[0].can_msg->id = can_get_mob_id(1,handle);
	pCANMOB_rx[0].dlc = can_get_mob_dlc(1,handle);
	pCANMOB_rx[0].status = event;
	can_mob_free(1,handle);
	//message_received_on_channel0 = true;
}


//! Boolean for message transmitted on CAN channel 0
volatile bool message_received_on_channel0 = false;
/*! \brief CAN Call Back when message is received
 */
void can_out_callback_channel0(U8 handle, U8 event)
{
	
	// Reception Only
	pCANMOB_message2[0].can_msg->data.u64 = can_get_mob_data(0,handle).u64;
	pCANMOB_message2[0].can_msg->id = can_get_mob_id(0,handle);
	pCANMOB_message2[0].dlc = can_get_mob_dlc(0,handle);
	pCANMOB_message2[0].status = event;
	can_mob_free(0,handle);
	message_received_on_channel0 = true;
}
void can_out_callback_channel00(U8 handle, U8 event)
{
	// Transmission Only
	can_mob_free(0,handle);
	message_transmitted_on_channel1 = true;
}
void can_task_init(void)
{
	// Setup the generic clock for CAN
	scif_gc_setup(AVR32_SCIF_GCLK_CANIF, SCIF_GCCTRL_OSC0, AVR32_SCIF_GC_NO_DIV_CLOCK, 0);
	// Now enable the generic clock
	scif_gc_enable(AVR32_SCIF_GCLK_CANIF);
	static const gpio_map_t CAN_GPIO_MAP = {
		{AVR32_CANIF_RXLINE_0_0_PIN, AVR32_CANIF_RXLINE_0_0_FUNCTION},
		{AVR32_CANIF_TXLINE_0_0_PIN, AVR32_CANIF_TXLINE_0_0_FUNCTION},
		{AVR32_CANIF_RXLINE_1_2_PIN, AVR32_CANIF_RXLINE_1_2_FUNCTION},
		{AVR32_CANIF_TXLINE_1_2_PIN, AVR32_CANIF_TXLINE_1_2_FUNCTION}
	};
	// Assign GPIO to CAN.
	gpio_enable_module(CAN_GPIO_MAP, sizeof(CAN_GPIO_MAP) / sizeof(CAN_GPIO_MAP[0]));
	start_can_scheduler();
}

/*! \brief CAN Task:
 *        - Check if messages are correctly transmitted/received
 *        - Update message for transmission/reception
 */
void can_task(void)
{
	/*
	if(ready_send)
	{
		//can_example_prepare_data_to_receive();
		//can_example_prepare_data_to_send();
		//can_example_prepare_data_to_send2();
		ready_send = false;
	}
	*/
	
}

/*! \brief CAN Prepare Data to Send
 *        - Allocate one MOB in transmission
 *        - Fill the MOB with the correct DATA
 *        - Start the transmission
 */
void can_example_prepare_data_to_send1(void)
{
	// Initialize channel 1
	can_init(1, ((U32)&mob_ram_ch1[0]), CANIF_CHANNEL_MODE_NORMAL, can_out_callback_channel1);
	// Allocate one mob for TX
	pCANMOB_message0[0].handle = can_mob_alloc(1);

	// Check return if no mob are available
	if (pCANMOB_message0[0].handle==CAN_CMD_REFUSED) 
	{
		while(true);
	}
	pCANMOB_message0[0].can_msg->data.u8[6] = 0;
	pCANMOB_message0[0].can_msg->data.u8[7] = counter1++;
	//pCANMOB_message0[0].can_msg->data.u8[1] = 0x87;
	can_tx(1, pCANMOB_message0[0].handle, pCANMOB_message0[0].dlc, pCANMOB_message0[0].req_type, pCANMOB_message0[0].can_msg);
	gpio_tgl_gpio_pin(LED1_GPIO);
	if(counter1 >= 63)
	{
		counter1 = 0;
	}
}

void can_example_prepare_data_to_receive1(void)
{
	// Initialize channel 0
	can_init(1, ((U32)&mob_ram_ch1[0]), CANIF_CHANNEL_MODE_NORMAL, can_out_callback_channel11);

	// Allocate one mob for RX
	pCANMOB_rx[0].handle = can_mob_alloc(1);
	// Check return if no mob are available
	if (pCANMOB_rx[0].handle== CAN_CMD_REFUSED)
	{
		while(true);
	}
	can_rx(1, pCANMOB_rx[0].handle, pCANMOB_rx[0].req_type, pCANMOB_rx[0].can_msg);
	gpio_tgl_gpio_pin(LED3_GPIO);
}




void can_example_prepare_data_to_send0(void)
{
	// Initialize channel 1
	can_init(0, ((U32)&mob_ram_ch0[0]), CANIF_CHANNEL_MODE_NORMAL, can_out_callback_channel00);
	// Allocate one mob for TX
	pCANMOB_message3C7[0].handle = can_mob_alloc(0);

	// Check return if no mob are available
	if (pCANMOB_message3C7[0].handle==CAN_CMD_REFUSED)
	{
		while(true);
	}
	pCANMOB_message3C7[0].can_msg->data.u8[6] = 0;
	pCANMOB_message3C7[0].can_msg->data.u8[7] = counter0++ ;
	//pCANMOB_message3C7[0].can_msg->data.u8[1] = 0x87;
	can_tx(0, pCANMOB_message3C7[0].handle, pCANMOB_message3C7[0].dlc, pCANMOB_message3C7[0].req_type, pCANMOB_message3C7[0].can_msg);
	gpio_tgl_gpio_pin(LED0_GPIO);
	if(counter0 >= 63)
	{
		counter0 = 0;
	}
}

/*! \brief CAN Prepare Data to Receive
 *        - Allocate one MOB in reception
 *        - Start the reception
 */
void can_example_prepare_data_to_receive0(void)
{
	// Initialize channel 0
	can_init(0, ((U32)&mob_ram_ch0[0]), CANIF_CHANNEL_MODE_NORMAL, can_out_callback_channel0);

	// Allocate one mob for RX
	pCANMOB_message2[0].handle = can_mob_alloc(0);
	// Check return if no mob are available
	if (pCANMOB_message2[0].handle== CAN_CMD_REFUSED)
	{
		while(true);
	}
	can_rx(0, pCANMOB_message2[0].handle, pCANMOB_message2[0].req_type, pCANMOB_message2[0].can_msg);
	gpio_tgl_gpio_pin(LED2_GPIO);
}


